// MIS3502 - Web Service Template
// Created by: Jeremy Shafer
// Fall 2023

// REMINDER - Don't forget to change your database connection
// timeout from 3 seconds to 3 minutes.
// Looku under Configuration / General Configuratio

// declarations *****************************************
const qs = require('qs'); //for parsing URL encoded data
const axios = require('axios'); // for calling another API
const base64 = require('js-base64'); // encode / decode
const mysql = require('mysql2/promise');  //for talking to a database

const dboptions = {
  'user' : '',
  'password' : '',
  'database' : '',
  'host' : 'dataanalytics.temple.edu'
};

//global connection variable
var connection;

const features = [
	"Issue a GET against dbtest and send someid=1 as a URL encoded key/value pair in the querystring. ",
	"Issue a GET against echotest and send some URL encoded key/value pairs in the querystring.  They will be echoed back as JSON.",
	"Issue a POST against echotest and send some URL encoded key/value pairs in the body.  They will be echoed back as JSON.",
	"Issue a PUT against echotest and send some URL encoded key/value pairs in the body.  They will be echoed back as JSON.",
	"Issue a PATCH against echotest and send some URL encoded key/value pairs in the body.  They will be echoed back as JSON.",
	"Issue a DELETE against echotest and send some URL encoded key/value pairs in the body.  They will be echoed back as JSON.",	
	];


// supporting functions *********************************

let dbtest = async (res,query) => {

	// get the expected variable
	let someid = query.someid;
	
	//error trap
	if (someid == undefined || isNaN(someid)){
		return formatres(res,"The key someid is missing or incorrect.",400);
	}
	
	//work and return the result
	let [result] = await connection.execute("select now() AS the_current_time, unix_timestamp(now()) AS the_current_epoch where 1 = ?",[someid]);
	return formatres(res,result,200);
}

let testFunction1 = (res,query) => {

	//error trap
	if (isEmpty(query)){
		return formatres(res,"That was a GET. No data sent",400);
	}

	//work and return the result
	return formatres(res,query,200);
}

let testFunction2 = (res,body) => {

	//error trap
	if (isEmpty(body)){
		return formatres(res,"That was a POST. No data sent",400);
	}
	
	//work and return the result	
	return formatres(res,body,200);
}

let testFunction3 = (res,body) => {
	if (isEmpty(body)){
		return formatres(res,"That was a PUT. No data sent",400);
	}
	
	//work and return the result	
	return formatres(res,body,200);
}

let testFunction4 = (res,body) => {
	if (isEmpty(body)){
		return formatres(res,"That was a PATCH. No data sent",400);
	}
	
	//work and return the result
	return formatres(res,body,200);
}

let testFunction5 = (res,body) => {
	if (isEmpty(body)){
		return formatres(res,"That was a DELETE. No data sent",400);
	}
	
	//work and return the result
	return formatres(res,body,200);
}

// do not delete this handy little supporting function
let formatres = (res, output, statusCode) => {
	
	// kill the global database connection
	if (connection != undefined &&  
		typeof(connection)=='object' &&  
		typeof(connection.end())=='object'  ){
		connection.end();
	}

	res.statusCode = statusCode;
	res.body = JSON.stringify(output);
	return res;	
}

// do not delete this handy little supportng function
function isEmpty(obj) {
    return Object.keys(obj).length === 0;
}

// My Routing Function **********************************

let myRoutingFunction = (res,method,path,query,body) => {

	// conditional statements go here.
	// look at the path and method and return the output from the 
	// correct supporting function.

	// Simple GET request with no features specified results
	// in a list of features / instructions
	if (method == "GET" && path == ""){
		return formatres(res, features, 200);
	}

	if (method == "GET" && path == "dbtest"){
		return dbtest(res,query);
	}

	if (method == "GET" && path == "echotest"){
		return testFunction1(res,query);
	}
	
	if (method == "POST" && path == "echotest"){
		return testFunction2(res,body);
	}
	
	if (method == "PUT" && path == "echotest"){
		return testFunction3(res,body);
	}
	
	if (method == "PATCH" && path == "echotest"){
		return testFunction4(res,body);
	}
	
	if (method == "DELETE" && path == "echotest"){
		return testFunction5(res,body);
	}
	
	return(res);
}


// event handler ****************************************

// Students should not have to change the code here.
// Students should be able to read and understand the code here.

exports.handler = async (request) => {

	connection = await mysql.createConnection(dboptions);	

	// identify the method (it will be a string)
	let method = request["httpMethod"];
	
	// identify the path (it will also be a string)
	let fullpath = request["path"];
	
	// we clean the full path up a little bit
	if (fullpath == undefined || fullpath == null){ fullpath = ""};
	let pathitems = fullpath.split("/");
	let path = pathitems[2];
	if (path == undefined || path == null){ path = ""};
	
	// identify the querystring ( we will convert it to 
	//   a JSON object named query)
	let query = request["queryStringParameters"];
	if (query == undefined || query == null){ query={} };
	
	// identify the body (we will convert it to 
	//   a JSON object named body)
	let body = qs.parse(request["body"]);
	if (body == undefined || body == null){ body={} };

	// Create the default response object that will include 
	// the status code, the headers needed by CORS, and
	// the string to be returned formatted as a JSON data structure.
    let res = {
        'statusCode': 400,
        'headers': {
            'Access-Control-Allow-Origin': '*',
            'Access-Control-Allow-Credentials': true  
        },
        'body': JSON.stringify("Feature not found."),
    };

	// run all the parameters through my routing function
	// and return the result
    return myRoutingFunction(res,method,path,query,body);
    
    //*** this is a good place to test one supporting function at a time
    //query={};
    //return dbtest(res,query);
    
};